(*       Greatis Form Designer Demo      *)
(*  Copyright (C) 2001 Greatis Software  *)
(*      web: http://www.greatis.com      *)
(*      e-mail: b-team@greatis.com       *)

unit Palette;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons, ExtCtrls, Clipbrd, Menus, FDMain;

type

  TControlType = (
    ctNone,
    ctLabel,
    ctEdit,
    ctMemo,
    ctButton,
    ctCheckBox,
    ctRadioButton,
    ctListBox,
    ctComboBox,
    ctGroupBox,
    ctPanel);

  TfrmPalette = class(TForm)
    pgcMain: TPageControl;
    tshMain: TTabSheet;
    sbtEditMode: TSpeedButton;
    sbtLabel: TSpeedButton;
    sbtEdit: TSpeedButton;
    sbtMemo: TSpeedButton;
    sbtButton: TSpeedButton;
    sbtCheckBox: TSpeedButton;
    sbtRadioButton: TSpeedButton;
    sbtListBox: TSpeedButton;
    sbtComboBox: TSpeedButton;
    sbtOpen: TSpeedButton;
    sbtSave: TSpeedButton;
    sbtGroupBox: TSpeedButton;
    sbtPanel: TSpeedButton;
    sbtAlignToGrid: TSpeedButton;
    sbtLock: TSpeedButton;
    sbtCut: TSpeedButton;
    sbtCopy: TSpeedButton;
    sbtPaste: TSpeedButton;
    sbtDelete: TSpeedButton;
    opdMain: TOpenDialog;
    svdMain: TSaveDialog;
    pmnMain: TPopupMenu;
    mniLock: TMenuItem;
    mniAlignToGrid: TMenuItem;
    mniCut: TMenuItem;
    mniCopy: TMenuItem;
    mniPaste: TMenuItem;
    mniDelete: TMenuItem;
    mniSep1: TMenuItem;
    sbtSetup: TSpeedButton;
    sbtAbout: TSpeedButton;
    mniBringToFront: TMenuItem;
    mniSendToBack: TMenuItem;
    mniSep2: TMenuItem;
    sbtAlign: TSpeedButton;
    sbtSize: TSpeedButton;
    sbtSelectAll: TSpeedButton;
    sbtAlignPalette: TSpeedButton;
    procedure sbtControlButtonClick(Sender: TObject);
    procedure sbtEditModeClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure eveLock(Sender: TObject);
    procedure eveAlign(Sender: TObject);
    procedure eveDelete(Sender: TObject);
    procedure eveCut(Sender: TObject);
    procedure evePaste(Sender: TObject);
    procedure eveCopy(Sender: TObject);
    procedure sbtOpenClick(Sender: TObject);
    procedure sbtSaveClick(Sender: TObject);
    procedure pmnMainPopup(Sender: TObject);
    procedure sbtSetupClick(Sender: TObject);
    procedure sbtAboutClick(Sender: TObject);
    procedure mniBringToFrontClick(Sender: TObject);
    procedure mniSendToBackClick(Sender: TObject);
    procedure sbtAlignClick(Sender: TObject);
    procedure sbtSizeClick(Sender: TObject);
    procedure eveSelectAll(Sender: TObject);
    procedure eveAlignPalette(Sender: TObject);
  private
    { Private declarations }
    procedure WMGetMinMaxInfo(var Msg: TMessage); message WM_GETMINMAXINFO;
  public
    { Public declarations }
    ControlType: TControlType;
  end;

var
  frmPalette: TfrmPalette;

implementation

uses Design, Setup, About;

{$R *.DFM}

procedure TfrmPalette.WMGetMinMaxInfo(var Msg: TMessage);
begin
  with PMinMaxInfo(Msg.lParam)^ do
  begin
    ptMinTrackSize.X:=Width;
    ptMinTrackSize.Y:=Height;
    ptMaxTrackSize:=ptMinTrackSize;
  end;
end;

procedure TfrmPalette.sbtControlButtonClick(Sender: TObject);
begin
  ControlType:=TControlType((Sender as TSpeedButton).Tag);
  with frmDesign.cmpFormDesigner do
    if not Locked then Lock;
end;

procedure TfrmPalette.sbtEditModeClick(Sender: TObject);
begin
  ControlType:=TControlType(0);
end;

procedure TfrmPalette.FormShow(Sender: TObject);
begin
  frmDesign.Show;
end;

procedure TfrmPalette.eveLock(Sender: TObject);
var
  i,IDX: Integer;
begin
  if Sender<>sbtLock then sbtLock.Down:=not sbtLock.Down;
  {$IFDEF TFD1COMPATIBLE}
  with frmDesign.cmpFormDesigner,FixedControls do
  {$ELSE}
  with frmDesign.cmpFormDesigner,LockedControls do
  {$ENDIF}
    for i:=0 to Pred(ControlCount) do
    begin
      IDX:=IndexOf(Controls[i].Name);
      if sbtLock.Down then
      begin
        if IDX=-1 then Add(Controls[i].Name);
      end
      else
        if IDX<>-1 then Delete(IDX);
    end;
end;

procedure TfrmPalette.pmnMainPopup(Sender: TObject);
begin
  mniLock.Checked:=sbtLock.Down;
end;

procedure TfrmPalette.eveAlign(Sender: TObject);
var
  i: Integer;
begin
  with frmDesign.cmpFormDesigner do
    for i:=0 to Pred(ControlCount) do
      if not IsLocked(Controls[i]) then
        AlignToGrid(Controls[i]);
end;

procedure TfrmPalette.eveDelete(Sender: TObject);
begin
  if sbtDelete.Enabled then
    with frmDesign.cmpFormDesigner do
      while ControlCount>0 do Controls[0].Free;
end;

procedure TfrmPalette.eveCut(Sender: TObject);
begin
  if sbtCut.Enabled then
    frmDesign.cmpFormDesigner.CutToClipboard;
end;

procedure TfrmPalette.evePaste(Sender: TObject);
begin
  if sbtPaste.Enabled then
    frmDesign.cmpFormDesigner.PasteFromClipboard;
end;

procedure TfrmPalette.eveCopy(Sender: TObject);
begin
  if sbtCopy.Enabled then
    frmDesign.cmpFormDesigner.CopyToClipboard;
end;

procedure TfrmPalette.sbtOpenClick(Sender: TObject);
begin
  with opdMain do
    if Execute then
      frmDesign.cmpFormDesigner.LoadFromDFM(FileName,TDFMFormat(Pred(FilterIndex)));
end;

procedure TfrmPalette.sbtSaveClick(Sender: TObject);
begin
  with svdMain do
    if Execute then
      frmDesign.cmpFormDesigner.SaveToDFM(FileName,TDFMFormat(Pred(FilterIndex)));
end;

procedure TfrmPalette.sbtSetupClick(Sender: TObject);
begin
  frmSetup.ShowModal;
end;

procedure TfrmPalette.sbtAboutClick(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

procedure TfrmPalette.mniBringToFrontClick(Sender: TObject);
var
  i: Integer;
begin
  with frmDesign.cmpFormDesigner do
  begin
    for i:=Pred(ControlCount) downto 0 do Controls[i].BringToFront;
    Update;
  end;
end;

procedure TfrmPalette.mniSendToBackClick(Sender: TObject);
var
  i: Integer;
begin
  with frmDesign.cmpFormDesigner do
  begin
    for i:=Pred(ControlCount) downto 0 do Controls[i].SendToBack;
    Update;
  end;
end;

procedure TfrmPalette.sbtAlignClick(Sender: TObject);
begin
  frmDesign.cmpFormDesigner.AlignDialog;
end;

procedure TfrmPalette.sbtSizeClick(Sender: TObject);
begin
  frmDesign.cmpFormDesigner.SizeDialog;
end;

procedure TfrmPalette.eveSelectAll(Sender: TObject);
begin
  frmDesign.cmpFormDesigner.SelectAll;
end;

procedure TfrmPalette.eveAlignPalette(Sender: TObject);
begin
  frmDesign.cmpFormDesigner.ShowAlignmentPalette;
end;

initialization
  RegisterClasses([TLabel,TEdit,TMemo,TButton,TCheckBox,TRadioButton,TListBox,TComboBox,TGroupBox,TPanel]);
end.
